
// gtrace.h

#ifndef _GTRACE_H_
#define _GTRACE_H_

#include <sys\timeb.h>  // for _timeb struct


// to compile GTRACE out of existence, add _GTRACE_DISABLE_
// to your project settings
#ifndef _GTRACE_DISABLE_
   #define GTRACE   GTrace xx_gtrace
#else
   #define GTRACE   (void)0
#endif



class GTrace
{
public:
   GTrace(DWORD dwFilter,      // "functionality" bit
          LPCTSTR lpszFormat,  // format string
          ...);                // optional parameters

protected:
   // call to output trace text
   virtual void Output(LPCTSTR lpszText,   // output text
                       int nLen);          // length of 'lpszText'

protected:
   // the time the GTRACE object was created, used to               
   // perform function timings
   struct _timeb m_startTime;
   // whether or not TRACEing is enabled
   BOOL m_bRun;
   // the trace output string
   CString m_strOutput;

private:
   // return time difference of two _timeb structs in milliseconds
   long DiffTime(_timeb startTime, _timeb stopTime);

public:
   virtual ~GTrace();
};





// registry keys/entries
#define GTRACEREG_KEY            _T("Software\\GTRACE")
#define GTRACEREG_ENTRY_FLAGS    _T("Flags")
#define GTRACEREG_ENTRY_FILTER   _T("Filter")
#define GTRACEREG_ENTRY_FILENAME _T("FileName")



typedef struct GTraceOptions
{
   CString strFileName;

   DWORD dwFlags;
   DWORD dwFilter;

} GTRACEOPTIONS, *LPGTRACEOPTIONS;



// trace flags...these are set by the user in the registry
#define TF_ENABLE          0x00000001  // enable tracing
#define TF_FUNCTIONTIMINGS 0x00000002  // perform function timings
#define TF_DEBUGOUTPUT     0x00000010  // force TRACE output to 
                                       // DEBUG window
#define TF_NOARGUMENTS     0x00000020  // ignore optional 
                                       // formatting parameters


// registry helpers
CString GRegQueryString(HKEY hSecKey, LPCTSTR lpszEntry, 
                                       LPCTSTR lpszDefault=NULL);
DWORD GRegQueryDword(HKEY hSecKey, LPCTSTR lpszEntry, 
                                              DWORD dwDefault=0);
BOOL GRegLoadTraceOptions(LPGTRACEOPTIONS lpTraceOptions);
BOOL GRegStoreTraceOptions(LPGTRACEOPTIONS lpTraceOptions);


#endif



