
// gtrcedlg.cpp

#include "stdafx.h"
#include "winreg.h"
#include "resource.h"
#include "gtrace.h"
#include "gtrcedlg.h"




///////////////////////////////////////////////////////////////////
// GTraceSettingsDlg dialog

GTraceSettingsDlg::GTraceSettingsDlg(CWnd* pParent) 
                     : CDialog(IDD_GTRACE_SETTINGS, pParent)
{
   m_bCheckEnable = FALSE;
   m_bCheckExcludeArgs = FALSE;
   m_bCheckFunctionTimings = FALSE;
   m_bCheckDebug = FALSE;

   m_lpTraceFilters = NULL;
   m_nFilterCount = 0;
}



void GTraceSettingsDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);

   // subclass list box
   DDX_Control(pDX, IDC_LIST_FILTER, m_listboxFilter);

   DDX_Check(pDX, IDC_CHECK_ENABLE, m_bCheckEnable);
   DDX_Check(pDX, IDC_CHECK_EXCLUDEARGS, m_bCheckExcludeArgs);
   DDX_Check(pDX, IDC_CHECK_FUNCTIMINGS, m_bCheckFunctionTimings);
   DDX_Check(pDX, IDC_CHECK_TRACE, m_bCheckDebug);

   DDX_Text(pDX, IDC_EDIT_FILENAME, m_strEditFileName);
   DDV_MaxChars(pDX, m_strEditFileName, 256);

   if(pDX->m_bSaveAndValidate)
   {
      GTRACEOPTIONS options;
      options.dwFilter = 0;
      options.dwFlags = 0;

      // set filter flags...
      int nCount = m_listboxFilter.GetCount();

      for(int i = 0; i < nCount; i++)
      {
         if(m_listboxFilter.GetCheck(i))
         {
            options.dwFilter |= m_listboxFilter.GetItemData(i);
         }
      }

      if(m_bCheckEnable)
         options.dwFlags |= TF_ENABLE;

      if(m_bCheckExcludeArgs)
         options.dwFlags |= TF_NOARGUMENTS;

      if(m_bCheckFunctionTimings)
         options.dwFlags |= TF_FUNCTIONTIMINGS;

      if(m_bCheckDebug)
         options.dwFlags |= TF_DEBUGOUTPUT;

      options.strFileName = m_strEditFileName;

      GRegStoreTraceOptions(&options);
   }
}



int GTraceSettingsDlg::DoModal(LPTRACEFILTER lpFilters, int nCount)
{
   m_lpTraceFilters = lpFilters;
   m_nFilterCount = nCount;

   ASSERT(m_lpTraceFilters && m_nFilterCount > 0);

   return CDialog::DoModal();
}


BEGIN_MESSAGE_MAP(GTraceSettingsDlg, CDialog)
   ON_BN_CLICKED(IDC_BUTTON_BROWSE, OnButtonBrowse)
   ON_BN_CLICKED(IDC_CHECK_TRACE, OnCheckTrace)
   ON_BN_CLICKED(IDC_CHECK_ENABLE, OnCheckEnable)
END_MESSAGE_MAP()




BOOL GTraceSettingsDlg::OnInitDialog()
{
   // load current registry settings
   GTRACEOPTIONS options;
   options.dwFlags = 0;
   options.dwFilter = 0;

   if(GRegLoadTraceOptions(&options))
   {
      m_bCheckEnable = (options.dwFlags & TF_ENABLE) 
                                                   ? TRUE : FALSE;
      m_bCheckExcludeArgs = (options.dwFlags & TF_NOARGUMENTS) 
                                                   ? TRUE : FALSE;
      m_bCheckFunctionTimings = 
            (options.dwFlags & TF_FUNCTIONTIMINGS) ? TRUE : FALSE;
      m_bCheckDebug = (options.dwFlags & TF_DEBUGOUTPUT) 
                                                   ? TRUE : FALSE;
      m_strEditFileName = options.strFileName;
   }

   CDialog::OnInitDialog();

   if(m_bCheckEnable)
   {
      EnableOutputFile(!m_bCheckDebug);
   }
   // otherwise, disable all controls
   else
   {
      EnableAll(m_bCheckEnable);
   }

   // populate list box
   m_listboxFilter.ResetContent();
   for(int i = 0; i < m_nFilterCount; i++)
   {
      m_listboxFilter.AddString(m_lpTraceFilters->lpszDesc);
      m_listboxFilter.SetItemData(i, m_lpTraceFilters->dwValue);

      if(options.dwFilter & m_lpTraceFilters->dwValue)
      {
         m_listboxFilter.SetCheck(i, 1);
      }

      m_lpTraceFilters++;
   }

   return TRUE;
}



void GTraceSettingsDlg::OnButtonBrowse() 
{
   CFileDialog Getfile(TRUE, NULL, NULL, OFN_NOVALIDATE);
                             
   Getfile.m_ofn.lpstrTitle = _T("Select output file");
   if(Getfile.DoModal() == IDOK) 
   {
      CWnd *pWnd = GetDlgItem(IDC_EDIT_FILENAME);
      if(pWnd)
         pWnd->SetWindowText(Getfile.GetPathName());
   }
}



void GTraceSettingsDlg::OnCheckTrace() 
{
   CButton *pButton = (CButton *)GetDlgItem(IDC_CHECK_TRACE);
   if(pButton)
      EnableOutputFile(pButton->GetCheck() == 0);
}



void GTraceSettingsDlg::EnableOutputFile(BOOL bEnable)
{
   CWnd *pWnd = GetDlgItem(IDC_EDIT_FILENAME);
   if(pWnd)
      pWnd->EnableWindow(bEnable);

   pWnd = GetDlgItem(IDC_BUTTON_BROWSE);
   if(pWnd)
      pWnd->EnableWindow(bEnable);
}


void GTraceSettingsDlg::OnCheckEnable() 
{
   CButton *pButton = (CButton *)GetDlgItem(IDC_CHECK_ENABLE);

   BOOL bEnable = pButton->GetCheck();
   EnableAll(bEnable);
   
   if(bEnable)
   {
      OnCheckTrace();
   }
}


void GTraceSettingsDlg::EnableAll(BOOL bEnable)
{
   int nID[] = {IDC_CHECK_TRACE, 
                IDC_EDIT_FILENAME,
                IDC_BUTTON_BROWSE,
                IDC_CHECK_EXCLUDEARGS,
                IDC_CHECK_FUNCTIMINGS,
                IDC_LIST_FILTER};

   int nCount = sizeof(nID)/sizeof(nID[0]);
   for(int i = 0; i < nCount; i++)
   {
      CWnd *pWnd = GetDlgItem(nID[i]);
      if(pWnd)
         pWnd->EnableWindow(bEnable);
   }
}


